"""Unittests for the pymp package."""
# pylint: disable=protected-access, invalid-name
from __future__ import print_function
import logging

import unittest

import pymp

logging.basicConfig(level=logging.INFO)


class ParallelTest(unittest.TestCase):

    """Test the parallel context."""

    def test_init(self):
        """Initialization test."""
        import pymp

        pymp.config.nested = False
        pymp.config.thread_limit = 4
        pinst = pymp.Parallel(2)
        with pinst as parallel:
            if not parallel._is_fork:
                self.assertEqual(len(parallel._pids), 1)
            nested_parallel = pymp.Parallel(2)
            self.assertRaises(AssertionError, nested_parallel.__enter__)
            pymp.config.nested = True
            with nested_parallel:
                pass
            pymp.config.nested = False
        self.assertRaises(AssertionError, pinst.__enter__)
        self.assertEqual(pymp.shared._NUM_PROCS.value, 1)
        self.assertEqual(pymp.Parallel._level, 0)

    @unittest.skipIf(
        not pymp._shared._NP_AVAILABLE, "Skipping test because numpy is not available."
    )
    def test_num_threads(self):
        """Test num threads property."""
        import pymp
        import os

        pymp.config.nested = False
        pymp.config.thread_limit = 4
        tlist = pymp.shared.list()
        with pymp.Parallel(2) as p:
            tlist.append(p.num_threads)
        self.assertEqual(list(tlist), [2, 2])
        pymp.config.nested = True
        tlist = pymp.shared.list()
        with pymp.Parallel(2) as p:
            with pymp.Parallel(2) as p2:
                tlist.append(p2.num_threads)
        self.assertEqual(list(tlist), [2, 2, 2, 2])

    def test_thread_num(self):
        """Test thread_num property."""
        import pymp

        pymp.config.nested = True
        pymp.config.thread_limit = 4
        tlist = pymp.shared.list()
        with pymp.Parallel(2) as p:
            tlist.append(p.thread_num)
        self.assertEqual(sorted(list(tlist)), [0, 1])
        tlist = pymp.shared.list()
        tlist2 = pymp.shared.list()
        tlist3 = pymp.shared.list()
        with pymp.Parallel(2) as p:
            with pymp.Parallel(2) as p2:
                if not p._is_fork:
                    tlist.append(p2.thread_num)
                else:
                    tlist2.append(p2.thread_num)
            tlist3.append(p.thread_num)
        self.assertEqual(sorted(list(tlist)), [0, 1])
        self.assertEqual(sorted(list(tlist2)), [0, 1])
        self.assertEqual(sorted(list(tlist3)), [0, 1])

    @unittest.skipIf(
        not pymp._shared._NP_AVAILABLE, "Skipping test because numpy is not available."
    )
    def test_range(self):
        """Range test."""
        import pymp

        pymp.config.nested = False
        pymp.config.thread_limit = 4
        try:
            import numpy as np
        except ImportError:
            return
        tarr = pymp.shared.array((5, 1))
        with pymp.Parallel(2) as p:
            for i in p.range(len(tarr)):
                tarr[i, 0] = 1.0
        self.assertEqual(np.sum(tarr), 5.0)

    @unittest.skipIf(
        not pymp._shared._NP_AVAILABLE, "Skipping test because numpy is not available."
    )
    def test_lock(self):
        """Lock test."""
        import pymp

        pymp.config.nested = False
        pymp.config.thread_limit = 4
        try:
            import numpy as np
        except ImportError:
            return
        tarr = pymp.shared.array((1, 1))
        lock = pymp.shared.lock()
        with pymp.Parallel(2) as p:
            for _ in p.range(1000):
                with lock:
                    tarr[0, 0] += 1.0
        self.assertEqual(tarr[0, 0], 1000.0)

    def test_list(self):
        """Shared list test."""
        import pymp

        pymp.config.nested = False
        pymp.config.thread_limit = 4
        tlist = pymp.shared.list()
        with pymp.Parallel(2) as p:
            for _ in p.range(1000):
                tlist.append(1.0)
        self.assertEqual(len(tlist), 1000)

    def test_dict(self):
        """Shared dict test."""
        import pymp

        pymp.config.nested = False
        pymp.config.thread_limit = 4
        tdict = pymp.shared.dict()
        with pymp.Parallel(2) as p:
            for iter_idx in p.range(400):
                tdict[iter_idx] = 1.0
        self.assertEqual(len(tdict), 400)

    def test_queue(self):
        """Shared queue test."""
        import pymp

        pymp.config.nested = False
        pymp.config.thread_limit = 4
        tqueue = pymp.shared.queue()
        with pymp.Parallel(2) as p:
            for iter_idx in p.range(400):
                tqueue.put(iter_idx)
        self.assertEqual(tqueue.qsize(), 400)

    def test_rlock(self):
        """Shared rlock test."""
        import pymp

        pymp.config.nested = False
        pymp.config.thread_limit = 4
        rlock = pymp.shared.rlock()
        tlist = pymp.shared.list()
        with pymp.Parallel(2):
            with rlock:
                with rlock:
                    tlist.append(1.0)
        self.assertEqual(len(tlist), 2)

    def test_thread_limit(self):
        """Thread limit test."""
        import pymp

        pymp.config.thread_limit = 3
        pymp.config.nested = True
        thread_list = pymp.shared.list()
        with pymp.Parallel(4) as p:
            thread_list.append(p.thread_num)
        thread_list = list(thread_list)
        thread_list.sort()
        self.assertEqual(list(thread_list), [0, 1, 2])
        thread_list = pymp.shared.list()
        with pymp.Parallel(2) as p:
            with pymp.Parallel(2) as p2:
                thread_list.append(p2.thread_num)
        thread_list = list(thread_list)
        thread_list.sort()
        self.assertTrue(thread_list == [0, 0, 1] or thread_list == [0, 0, 1, 1])
        # Second case if the first two threads were exiting already.

    def test_xrange(self):
        """Test the dynamic schedule."""
        import pymp

        pymp.config.thread_limit = 4
        pymp.config.nested = True
        tlist = pymp.shared.list()
        with pymp.Parallel(2):
            with pymp.Parallel(2) as p:
                for idx in p.xrange(5):
                    tlist.append(idx)
        self.assertEqual(len(tlist), 10)

    def test_exceptions(self):
        """Test raising behavior."""
        import pymp

        pymp.config.thread_limit = 4
        pymp.config.nested = True

        def exc_context():
            """Creates a context with an Exception in a subthread."""
            with pymp.Parallel(2) as p:
                if p.thread_num == 1:
                    raise Exception()

        self.assertRaises(Exception, exc_context)

    def test_print(self):  # pylint: disable=no-self-use
        """Test the print method."""
        import pymp

        pymp.config.thread_limit = 3
        pymp.config.nested = True
        with pymp.Parallel(2):
            with pymp.Parallel(2) as p:
                p.print("Hi from thread {0}.".format(p.thread_num))

    def test_safety_check(self):
        """Test that the methods can only be used within their context."""
        import pymp

        pymp.config.thread_limit = 3
        pymp.config.nested = True
        p = pymp.Parallel(2)
        # Exception before use.
        self.assertRaises(AssertionError, lambda: p.thread_num)
        self.assertRaises(AssertionError, lambda: p.num_threads)
        self.assertRaises(AssertionError, lambda: p.lock)
        self.assertRaises(AssertionError, lambda: p.range(10))
        self.assertRaises(AssertionError, lambda: p.xrange(10))
        with p:
            pass
        # Exception after use.
        self.assertRaises(AssertionError, lambda: p.thread_num)
        self.assertRaises(AssertionError, lambda: p.num_threads)
        self.assertRaises(AssertionError, lambda: p.lock)
        self.assertRaises(AssertionError, lambda: p.range(10))
        self.assertRaises(AssertionError, lambda: p.xrange(10))

    def test_if(self):
        """Test the if_ deactivation."""
        import pymp

        pymp.config.thread_limit = 3
        pymp.config.nested = True
        with pymp.Parallel(if_=False) as p:
            self.assertEqual(p.num_threads, 1)

    @unittest.skipIf(
        not pymp._shared._NP_AVAILABLE, "Skipping test because numpy is not available."
    )
    def test_noreshape(self):
        """Test if reshaping is effectively prevented."""
        import pymp

        sa = pymp.shared.array((3, 3))
        self.assertRaises(ValueError, lambda: sa.reshape((4, 4)))
        sa.reshape((1, 3, 3))

    def test_iterable_two_threads(self):
        """Test if iterating over an iterable is working correctly."""
        import pymp

        rnge = iter(range(10))
        thread_list = pymp.shared.list()
        with pymp.Parallel(2) as p:
            for elem in p.iterate(rnge):
                thread_list.append((p.thread_num, elem))
        elements = [item[1] for item in thread_list]
        self.assertEqual(sorted(elements), list(range(10)))
        threads = [item[0] for item in thread_list]
        for item in threads:
            self.assertEqual(item, 1)

    def test_iterable_one_thread(self):
        """Test if iterating over an iterable is working correctly."""
        import pymp

        rnge = iter(range(10))
        thread_list = pymp.shared.list()
        with pymp.Parallel(1) as p:
            for elem in p.iterate(rnge):
                thread_list.append((p.thread_num, elem))
        elements = [item[1] for item in thread_list]
        self.assertEqual(sorted(elements), list(range(10)))
        threads = [item[0] for item in thread_list]
        for item in threads:
            self.assertEqual(item, 0)

    def test_iterable_three_threads(self):
        """Test if iterating over an iterable is working correctly."""
        import pymp

        pymp.config.thread_limit = 3
        rnge = iter(range(10))
        thread_list = pymp.shared.list()
        with pymp.Parallel(3) as p:
            for elem in p.iterate(rnge):
                thread_list.append((p.thread_num, elem))
        elements = [item[1] for item in thread_list]
        self.assertEqual(sorted(elements), list(range(10)))
        threads = [item[0] for item in thread_list]
        for item in threads:
            self.assertTrue(item in [1, 2])


if __name__ == "__main__":
    unittest.main()



"""Configuration."""
# pylint: disable=invalid-name
from __future__ import print_function

import logging as _logging
import multiprocessing as _multiprocessing
import os as _os

_LOGGER = _logging.getLogger(__name__)


def _get_conf_value(suffix):
    """Get configuration value from PYMP/OMP env variables."""
    pymp_name = "PYMP_" + suffix
    omp_name = "OMP_" + suffix
    value = None
    for env_name in [pymp_name, omp_name]:
        # pylint: disable=no-member
        if env_name in _os.environ:
            _LOGGER.debug(
                "Using %s environment variable: %s.", env_name, _os.environ[env_name]
            )
            value = _os.environ[env_name]
            break
    return value


# Initialize configuration.
_nested_env = _get_conf_value("NESTED")
if _nested_env is None:
    #: Whether nesting of parallel sections is allowed.
    nested = False
else:  # pragma: no cover
    assert _nested_env.lower() in ["true", "false"], (
        "The configuration for PYMP_NESTED/OMP_NESTED must be either "
        "TRUE or FALSE. Is %s.",
        _nested_env,
    )
    nested = _nested_env.lower() == "true"

_num_threads_env = _get_conf_value("NUM_THREADS")
if _num_threads_env is None:  # pragma: no cover
    #: The number of threads to use as default. Defaults to
    #: CPU count.
    # pylint: disable=no-member
    num_threads = [_multiprocessing.cpu_count()]
else:
    _num_threads_env = [int(_val) for _val in _num_threads_env.split(",")]
    for _val in _num_threads_env:
        assert _val > 0, (
            "The PYMP_NUM_THREADS/OMP_NUM_THREADS variable must be a comma "
            "separated list of positive integers, specifying the number "
            "of threads to use in each nested level."
        )
    num_threads = _num_threads_env[:]

_thread_limit_env = _get_conf_value("THREAD_LIMIT")
if _thread_limit_env is None:  # pragma: no cover
    #: The thread limit to use. This is the maximum number of TOTAL threads
    #: in use independent of nesting. Even if more threads are requested,
    #: the maximum can not be exceeded.
    thread_limit = None
else:
    thread_limit = int(_thread_limit_env)
    assert thread_limit > 0, (
        "The PYMP_THREAD_LIMIT/OMP_THREAD_LIMIT variable must be an intereger "
        "greater zero!"
    )



"""Main package."""
# pylint: disable=invalid-name, bad-continuation, len-as-condition
from __future__ import print_function

import functools as _functools
import logging as _logging
import multiprocessing as _multiprocessing
import os as _os
import platform as _platform
import sys as _sys
import time as _time

import pymp.config as _config
import pymp.shared as _shared

_is_py2 = _sys.version[0] == "2"
if _is_py2:
    import Queue as _Queue  # pylint: disable=import-error
else:
    import queue as _Queue  # pylint: disable=import-error, E0102


_LOGGER = _logging.getLogger(__name__)


# pylint: disable=too-few-public-methods, too-many-instance-attributes
class Parallel(object):

    """A parallel region."""

    _level = 0
    _global_master = None

    def __init__(
        self, num_threads=None, if_=True
    ):  # pylint: disable=redefined-outer-name
        if _platform.system().startswith("Windows") or _platform.system().startswith(
            "CYGWIN"
        ):
            raise Exception(
                "Pymp relies on full 'fork' support by the operating system. "
                "You seem to be using Windows, which unfortanetly does not "
                "do this."
            )
        self._num_threads = num_threads
        self._enabled = if_
        if not self._enabled:
            self._num_threads = 1
        self._is_fork = False
        self._pids = []
        self._thread_num = 0
        self._lock = None
        if Parallel._global_master is None:
            Parallel._global_master = _os.getpid()
        # Dynamic schedule management.
        self._dynamic_queue = _shared.queue()
        self._iter_queue = None
        self._thread_loop_ids = None
        self._queuelock = _shared.lock()
        # Exception management.
        self._exception_queue = _shared.queue()
        self._exception_lock = _shared.lock()
        # Allows for self-checks.
        self._entered = False
        self._disposed = False

    def __enter__(self):
        _LOGGER.debug(
            "Entering `Parallel` context (level %d). Forking...", Parallel._level
        )
        # pylint: disable=global-statement
        assert len(self._pids) == 0, "A `Parallel` object may only be used once!"
        self._lock = _shared.lock()
        # pylint: disable=protected-access
        if self._num_threads is None:
            assert (
                len(_config.num_threads) == 1
                or len(_config.num_threads) > Parallel._level
            ), (
                "The value of PYMP_NUM_THREADS/OMP_NUM_THREADS must be "
                "either a single positive number or a comma-separated "
                "list of number per nesting level."
            )
            if len(_config.num_threads) == 1:
                self._num_threads = _config.num_threads[0]
            else:
                self._num_threads = _config.num_threads[Parallel._level]
        if not _config.nested:
            assert Parallel._level == 0, "No nested parallel contexts allowed!"
        Parallel._level += 1
        self._iter_queue = _shared.queue(maxsize=self._num_threads - 1)
        # pylint: disable=protected-access
        with _shared._LOCK:
            # Make sure that max threads is not exceeded.
            if _config.thread_limit is not None:
                # pylint: disable=protected-access
                num_active = _shared._NUM_PROCS.value
                self._num_threads = min(
                    self._num_threads, _config.thread_limit - num_active + 1
                )
            _shared._NUM_PROCS.value += self._num_threads - 1
        self._thread_loop_ids = _shared.list([-1] * self._num_threads)
        for thread_num in range(1, self._num_threads):
            pid = _os.fork()
            if pid == 0:  # pragma: no cover
                # Forked process.
                self._is_fork = True
                self._thread_num = thread_num
                break
            else:
                # pylint: disable=protected-access
                self._pids.append(pid)
        if not self._is_fork:
            _LOGGER.debug("Forked to processes: %s.", str(self._pids))
        self._entered = True
        return self

    def __exit__(self, exc_t, exc_val, exc_tb):
        _LOGGER.debug("Leaving parallel region (%d)...", _os.getpid())
        if exc_t is not None:
            with self._exception_lock:
                self._exception_queue.put((exc_t, exc_val, self._thread_num))
        if self._is_fork:  # pragma: no cover
            _LOGGER.debug("Process %d done. Shutting down.", _os.getpid())
            _os._exit(1)  # pylint: disable=protected-access
        for pid in self._pids:
            _LOGGER.debug("Waiting for process %d...", pid)
            _os.waitpid(pid, 0)
        # pylint: disable=protected-access
        with _shared._LOCK:
            _shared._NUM_PROCS.value -= len(self._pids)
        Parallel._level -= 1
        if _os.getpid() == Parallel._global_master:
            # Reset the manager object.
            # pylint: disable=protected-access, no-member
            _shared._MANAGER = _multiprocessing.Manager()
        self._disposed = True
        # Take care of exceptions if necessary.
        if self._enabled:
            while not self._exception_queue.empty():
                exc_t, exc_val, thread_num = self._exception_queue.get()
                _LOGGER.critical(
                    "An exception occured in thread %d: (%s, %s).",
                    thread_num,
                    exc_t,
                    exc_val,
                )
                raise exc_t(exc_val)
        else:
            if not self._exception_queue.empty():
                raise
        _LOGGER.debug("Parallel region left (%d).", _os.getpid())

    def _assert_active(self):
        """Assert that the parallel region is active."""
        assert (
            self._entered and not self._disposed
        ), "The parallel context of this object is not active!"

    @property
    def thread_num(self):
        """The worker index."""
        self._assert_active()
        return self._thread_num

    @property
    def num_threads(self):
        """The number of threads in this context."""
        self._assert_active()
        return self._num_threads

    @property
    def lock(self):
        """Get a convenient, context specific lock."""
        self._assert_active()
        return self._lock

    @classmethod
    def print(cls, *args, **kwargs):
        """Print synchronized."""
        # pylint: disable=protected-access
        with _shared._PRINT_LOCK:
            print(*args, **kwargs)
            _sys.stdout.flush()

    def range(self, start, stop=None, step=1):
        """
        Get the correctly distributed parallel chunks.

        This corresponds to using the OpenMP 'static' schedule.
        """
        self._assert_active()
        if stop is None:
            start, stop = 0, start
        full_list = range(start, stop, step)
        per_worker = len(full_list) // self._num_threads
        rem = len(full_list) % self._num_threads
        schedule = [
            per_worker + 1 if thread_idx < rem else per_worker
            for thread_idx in range(self._num_threads)
        ]
        # pylint: disable=undefined-variable
        start_idx = _functools.reduce(
            lambda x, y: x + y, schedule[: self.thread_num], 0
        )
        end_idx = start_idx + schedule[self._thread_num]
        return full_list[start_idx:end_idx]

    def xrange(self, start, stop=None, step=1):
        """
        Get an iterator for this threads chunk of work.

        This corresponds to using the OpenMP 'dynamic' schedule.
        """
        self._assert_active()
        if stop is None:
            start, stop = 0, start
        with self._queuelock:
            pool_loop_reached = max(self._thread_loop_ids)
            # Get this loop id.
            self._thread_loop_ids[self._thread_num] += 1
            loop_id = self._thread_loop_ids[self._thread_num]
            if pool_loop_reached < loop_id:
                # No thread reached this loop yet. Set up the queue.
                for idx in range(start, stop, step):
                    self._dynamic_queue.put(idx)
            # Iterate.
            return _QueueIterator(self._dynamic_queue, loop_id, self)

    def iterate(self, iterable, element_timeout=None):
        """
        Iterate over an iterable.

        The iterator is executed in the host thread. The threads dynamically
        grab the elements. The iterator elements must hence be picklable to
        be transferred through the queue.

        If there is only one thread, no special operations are performed.
        Otherwise, effectively n-1 threads are used to process the iterable
        elements, and the host thread is used to provide them.

        You can specify a timeout for the clients to adhere.
        """
        self._assert_active()
        with self._queuelock:
            # Get this loop id.
            self._thread_loop_ids[self._thread_num] += 1
            loop_id = self._thread_loop_ids[self._thread_num]
            # Iterate.
            return _IterableQueueIterator(
                self._iter_queue, loop_id, self, iterable, element_timeout
            )


class _QueueIterator(object):

    """Iterator to create the dynamic schedule."""

    def __init__(self, queue, loop_id, pcontext):
        self._queue = queue
        self._loop_id = loop_id
        self._pcontext = pcontext

    # pylint: disable=non-iterator-returned
    def __iter__(self):
        return self

    def __next__(self):
        return self.next()

    def next(self):
        """Iterator implementation."""
        # pylint: disable=protected-access
        with self._pcontext._queuelock:
            # Check that the pool still deals with this loop.
            # pylint: disable=protected-access
            pool_loop_reached = max(self._pcontext._thread_loop_ids)
            if pool_loop_reached > self._loop_id or self._queue.empty():
                raise StopIteration()
            else:
                return self._queue.get()


class _IterableQueueIterator(object):

    """Iterator for the iterable queue."""

    def __init__(  # pylint: disable=too-many-arguments
        self, queue, loop_id, pcontext, iterable, element_timeout
    ):
        self._queue = queue
        self._loop_id = loop_id
        self._pcontext = pcontext
        if self._pcontext.thread_num == 0:
            self._iterable = iterable
            self._element_timeout = element_timeout

    # pylint: disable=non-iterator-returned
    def __iter__(self):
        if self._pcontext.num_threads == 1:
            return iter(self._iterable)
        return self

    def __next__(self):
        return self.next()

    def next(self):
        """Iterator implementation."""
        # pylint: disable=protected-access
        while True:
            if self._pcontext.thread_num == 0 and self._pcontext.num_threads > 1:
                # Producer.
                for iter_elem in self._iterable:
                    self._queue.put(iter_elem, timeout=self._element_timeout)
                for _ in range(self._pcontext.num_threads - 1):
                    self._queue.put(
                        "__queueend__:%d" % (self._pcontext._thread_loop_ids[0])
                    )
                raise StopIteration()
            elif self._pcontext.num_threads > 1:
                # Consumer.
                # Check that the pool still deals with this loop.
                # pylint: disable=protected-access
                with self._pcontext._queuelock:
                    pool_loop_reached = max(self._pcontext._thread_loop_ids)
                    master_reached = self._pcontext._thread_loop_ids[0]
                if pool_loop_reached > self._loop_id:
                    raise StopIteration()
                elif master_reached < self._loop_id:
                    # The producer did not reach this loop yet.
                    _time.sleep(0.1)
                    continue
                else:
                    try:
                        queue_elem = self._queue.get(timeout=0.1)
                    except _Queue.Empty:
                        continue
                    if queue_elem == "__queueend__:%d" % self._loop_id:
                        raise StopIteration()
                    else:
                        return queue_elem
            else:
                # Single thread execution.
                # Should have never reached here, since this case is dealt with
                # in the __iter__ method!
                raise Exception("Internal error!")



"""Contains shared datastructures."""
# pylint: disable=no-member, unused-import, ungrouped-imports
import multiprocessing as _multiprocessing
import warnings as _warnings
from multiprocessing import Lock as lock
from multiprocessing import RLock as rlock

try:
    import numpy as _np

    _NP_AVAILABLE = True
except ImportError:  # pragma: no cover
    _NP_AVAILABLE = False

_MANAGER = _multiprocessing.Manager()  # pylint: disable=no-member
_NUM_PROCS = _multiprocessing.Value("i", 1, lock=False)  # pylint: disable=no-member
_LOCK = lock()
_PRINT_LOCK = lock()


def array(shape, dtype=None, autolock=False):
    """
    Factory method for shared memory arrays supporting all numpy dtypes.

    By default the dtype will be set to float64 (if dtype is None).
    """
    assert _NP_AVAILABLE, "To use the shared array object, numpy must be available!"
    if dtype is None:
        dtype = _np.float64
    if not isinstance(dtype, _np.dtype):
        dtype = _np.dtype(dtype)
    # Not bothering to translate the numpy dtypes to ctype types directly,
    # because they're only partially supported. Instead, create a byte ctypes
    # array of the right size and use a view of the appropriate datatype.
    shared_arr = _multiprocessing.Array(
        "b", int(_np.prod(shape) * dtype.itemsize), lock=autolock
    )
    with _warnings.catch_warnings():
        # For more information on why this is necessary, see
        # https://www.reddit.com/r/Python/comments/j3qjb/parformatlabpool_replacement
        _warnings.simplefilter("ignore", RuntimeWarning)
        data = _np.ctypeslib.as_array(shared_arr).view(dtype).reshape(shape)
    return data


def list(*args, **kwargs):  # pylint: disable=redefined-builtin
    """Create a shared list."""
    return _MANAGER.list(*args, **kwargs)


def dict(*args, **kwargs):  # pylint: disable=redefined-builtin
    """Create a shared dict."""
    return _MANAGER.dict(*args, **kwargs)


def queue(*args, **kwargs):
    """Create a shared queue."""
    return _MANAGER.Queue(*args, **kwargs)



# -*- coding: utf-8 -*-
"""
The setup script for the entire project.
@author: Christoph Lassner
"""
from setuptools import find_packages, setup

VERSION = "0.4.5"

setup(
    name="pymp-pypi",
    author="Christoph Lassner",
    author_email="mail@christophlassner.de",
    packages=find_packages(),
    version=VERSION,
    test_suite="tests.unittests",
    license="MIT License",
    url="https://github.com/classner/pymp",
    download_url="https://github.com/classner/pymp/tarball/v{0}".format(VERSION),
    keywords=["multiprocessing", "openmp", "parallelism"],
)
